<?php

namespace Drupal\harmonize\Service;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Service used in Refiners & Harmony files to access common helper functions.
 *
 * @package Drupal\harmonize\Service
 */
class Harmonics {

  /**
   * The Cache service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cacheBackend;

  /**
   * Use DI to inject Drupal's configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  public $configFactory;

  /**
   * The Date Formatter object.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The Entity Type Manager object.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The Harmonize service.
   *
   * @var \Drupal\harmonize\Service\Harmonize
   */
  protected $harmonize;

  /**
   * The Image Style storage.
   *
   * @var \Drupal\image\ImageStyleStorageInterface
   */
  protected $imageStyleStorage;

  /**
   * Groomer Manager injected through DI.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The Drupal Logger Channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $loggerChannel;

  /**
   * Service that contains all functionality for the harmonizer module.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBackend
   *   The Cache service injected through DI.
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   *   Configuration Factory service injected through DI.
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   *   The Date Formatter object.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type manager injected through DI.
   * @param \Drupal\harmonize\Service\Harmonize $harmonize
   *   The groomer manager, injected through DI.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   Language manager injected through DI.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerChannelFactory
   *   Logger Channel factory injected through DI.
   */
  public function __construct(
    CacheBackendInterface $cacheBackend,
    ConfigFactory $configFactory,
    DateFormatter $dateFormatter,
    EntityTypeManagerInterface $entityTypeManager,
    Harmonize $harmonize,
    LanguageManagerInterface $languageManager,
    LoggerChannelFactoryInterface $loggerChannelFactory
  ) {
    $this->cacheBackend = $cacheBackend;
    $this->configFactory = $configFactory;
    $this->dateFormatter = $dateFormatter;
    $this->entityTypeManager = $entityTypeManager;
    $this->harmonize = $harmonize;
    $this->languageManager = $languageManager;
    $this->loggerChannel = $loggerChannelFactory->get('harmonics');
    try {
      $this->imageStyleStorage = $this->entityTypeManager->getStorage(
        'image_style'
      );
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      $this->loggerChannel->error("Unable to load Image Style Storage.");
    }
  }

  /**
   * Format an image into a standardized object for fractal templates.
   *
   * @param array $imageData
   *   The image data obtained from Harmonize.
   * @param string|null $style
   *   Image style to format the image with.
   *
   * @return array
   *   Formatted image data.
   */
  public function niceImage(array $imageData, string $style = NULL) : ?array {
    // Set the default image source.
    $src = $imageData['attributes']['src'];

    // Get image style if we must.
    if ($style) {
      /** @var \Drupal\image\Entity\ImageStyle $imageStyle */
      $imageStyle = $this->imageStyleStorage->load($style);

      if ($imageStyle === NULL) {
        $this->loggerChannel->warning('The image style "' . $style . '" doesn\'t exist.');
      }
      else {
        $src = $imageStyle->buildUrl($imageData['uri']);
      }

    }

    return [
      'alt' => $imageData['attributes']['alt'],
      'src' => $src,
    ];
  }

  /**
   * Format a date into a standardized object appropriate for fractal templates.
   *
   * @param string|array $date
   *   The original date string.
   * @param string $format
   *   Specify the format the date label should be displayed in.
   *
   * @return array
   *   Date object containing label and readable values.
   */
  public function niceDate($date, string $format): ?array {
    // Handle date ranges.
    if (is_array($date) && isset($date['from'], $date['to'])) {
      $from = $this->niceDate($date['from'], $format);
      $to = $this->niceDate($date['to'], $format);

      // If they're on the same day, we format our label differently.
      if ($from['readable'] === $to['readable']) {
        return [
          'label' => $from['label'] . ' | ' . $from['time'] . ' - ' . $to['time'] . ' ' . $from['timezone'],
          'readable' => $from['readable'],
        ];
      }

      return [
        'label' => $from['label'] . ' ' . $from['time'] . ' - ' . $to['label'] . ' ' . $to['time'] . ' ' . $from['timezone'],
        'readable' => $from['readable'],
      ];
    }

    return [
      'label' => $this->dateFormatter->format(strtotime($date), 'custom', $format, date_default_timezone_get()),
      'readable' => $this->dateFormatter->format(strtotime($date), 'custom', 'Y-m-d', date_default_timezone_get()),
      'time' => $this->dateFormatter->format(strtotime($date), 'custom', 'h:i a', date_default_timezone_get()),
      'timezone' => $this->dateFormatter->format(strtotime($date), 'custom', 'T', date_default_timezone_get()),
    ];
  }

  /**
   * Format a link into a standardized object appropriate for fractal templates.
   *
   * @param array $linkFieldConsignment
   *   Consigned data of the link field.
   * @param array $overrides
   *   Used to override attributes of the link field.
   *
   * @return array
   *   Array of normalized consistent data of the link.
   */
  public function niceLink(array $linkFieldConsignment, array $overrides = []): ?array {
    if (!$linkFieldConsignment) {
      return NULL;
    }

    $link = [
      'href' => $linkFieldConsignment['attributes']['href'] ?? NULL,
      'text' => $linkFieldConsignment['label'] ?? NULL,
      'target' => $linkFieldConsignment['attributes']['target'] ?? '_self',
    ];

    if ($overrides) {
      $link = array_merge($link, $overrides);
    }

    if ($link['target'] === '_blank') {
      $link['rel'] = 'noopener noreferrer';
    }

    return $link;
  }

}
